@@ -16,7 +16,107 @@ import org.junit.Ignore;
 import org.junit.Test;
 
 public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
+	
+	// test bug #65
+	@Test
+	public void nbspTest() throws Exception{
+		String html="<span>Change text size&nbsp;</span>";
+		String expected="<span>Change text size\u00A0</span>";
+    	cleaner.getProperties().setOmitXmlDeclaration(false);
+    	cleaner.getProperties().setAddNewlineToHeadAndBody(false);
+		assertHTML(expected, html);
+	}
+	
+	// Test bug #196. Doesn't currently pass.
+	@Ignore 
+	@Test
+	public void emptyString() throws Exception{
+		String html = " ";
+		
+		CleanerProperties props = new CleanerProperties();
+	    props.setOmitXmlDeclaration(true);
+	    props.setOmitHtmlEnvelope(true);
+	    
+	    HtmlCleaner cleaner = new HtmlCleaner(props);
+	    TagNode rootTag = cleaner.clean(html);
+	    XmlSerializer serializer = new SimpleXmlSerializer(props);
+	    ContentNode contentNode = (ContentNode)rootTag.getAllChildren().get(0);
 
+	    String cleanHtml = serializer.getAsString(rootTag, false);
+	    assertEquals(" ", contentNode.content);
+	    assertEquals(" ", cleanHtml);
+	}
+	
+	@Test
+	public void linefeedsInElements() throws IOException {
+		HtmlCleaner theCleaner = new HtmlCleaner();
+		theCleaner.getProperties().setRecognizeUnicodeChars(false);
+		theCleaner.getProperties().setOmitHtmlEnvelope(true);
+		theCleaner.getProperties().setOmitXmlDeclaration(true);
+		String initial = "<dfn>something: &#xA;</dfn>";
+		String expected = "<dfn>something: &#xA;</dfn>";
+		String cleaned = new SimpleHtmlSerializer(theCleaner.getProperties()).getAsString(theCleaner.clean(initial));
+	    assertEquals(cleaned, expected);
+		cleaned = new SimpleXmlSerializer(theCleaner.getProperties()).getAsString(theCleaner.clean(initial));
+	    assertEquals(cleaned, expected);
+	}
+	
+	@Test
+	public void operatorsInAttributes() throws IOException {
+		HtmlCleaner theCleaner = new HtmlCleaner();
+		theCleaner.getProperties().setAllowHtmlInsideAttributes(true);
+		theCleaner.getProperties().setOmitXmlDeclaration(true);
+		String initial = "<dfn title=\"something is < 100\">";
+		String expected = "<html><head></head><body><dfn title=\"something is &lt; 100\"></dfn></body></html>";
+		String cleaned = new SimpleHtmlSerializer(theCleaner.getProperties()).getAsString(theCleaner.clean(initial));
+	    assertEquals(cleaned, expected);
+	}
+	
+	@Test
+	public void operatorsInAttributes2() throws IOException {
+		String initial = "<dfn title=\"something is &lt; 100\">";
+		String expected = "<html><head></head><body><dfn title=\"something is &lt; 100\"></dfn></body></html>";
+		assertCleanedHtml(initial, expected);
+	}
+	
+	@Test
+	public void operatorsInAttributes3() throws IOException {
+		HtmlCleaner theCleaner = new HtmlCleaner();
+		theCleaner.getProperties().setAllowHtmlInsideAttributes(false);
+		theCleaner.getProperties().setOmitXmlDeclaration(true);
+		String initial = "<dfn title=\"something is < 100\">";
+		String expected = "<html><head></head><body><dfn title=\"something is\">&lt; 100&quot;&gt;</dfn></body></html>";
+		String cleaned = new SimpleHtmlSerializer(theCleaner.getProperties()).getAsString(theCleaner.clean(initial));
+	    assertEquals(cleaned, expected);
+	}
+	
+	@Test
+	public void operatorsInAttributes4() throws IOException {
+		HtmlCleaner theCleaner = new HtmlCleaner();
+		theCleaner.getProperties().setAllowHtmlInsideAttributes(false);
+		theCleaner.getProperties().setOmitXmlDeclaration(true);
+		String initial = "<dfn title=\"something is &lt; 100\">";
+		String expected = "<html><head></head><body><dfn title=\"something is &lt; 100\"></dfn></body></html>";
+		String cleaned = new SimpleHtmlSerializer(theCleaner.getProperties()).getAsString(theCleaner.clean(initial));
+	    assertEquals(cleaned, expected);
+	}
+	
+	@Test
+	public void escaped() throws IOException{
+		HtmlCleaner thecleaner = new HtmlCleaner();
+		String initial = readFile("src/test/resources/escaped.html");
+		initial = initial.replaceAll("&lt;", "<");
+		initial = initial.replaceAll("&gt;", ">");
+		TagNode node = thecleaner.clean(initial);
+		new SimpleHtmlSerializer(thecleaner.getProperties()).getAsString(node);
+	}
+
+	@Test
+	public void unescapedtagtokens() throws IOException{
+		String initial = "<div id=\"1234\">My name is ABC and my age is < 30. </div>";
+		String expected = "<html><head></head><body><div id=\"1234\">My name is ABC and my age is &lt; 30. </div></body></html>";
+		assertCleanedHtml(initial, expected);
+	}
 	
 	@Test
 	public void href() throws IOException{
@@ -112,6 +212,24 @@ public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
 
 	}
 	
+	/**
+	 * Element names in HTML and XML
+	 */
+	@Test
+	public void elementNames() throws Exception {
+	    cleaner.getProperties().setOmitHtmlEnvelope(true);
+	    cleaner.getProperties().setNamespacesAware(true);
+	    
+	    assertCleanedDom("<p><^-^></p>", "<p>&amp;lt;^-^&amp;gt;</p>");
+	    assertCleanedJDom("<p><^-^></p>", "<p>&amp;lt;^-^&amp;gt;</p>");
+	    assertCleanedDom("<p><1o></p>", "<p>&amp;lt;1o&amp;gt;</p>");
+	    assertCleanedJDom("<p><1o></p>", "<p>&amp;lt;1o&amp;gt;</p>");
+	    assertCleanedDom("<p><b=></b=></p>", "<p>\n<b></b>\n</p>");
+	    assertCleanedJDom("<p><b=></b=></p>", "<p><b /></p>");
+	    assertCleanedDom("<p><b^b></b=></p>", "<p>\n<bb></bb>\n</p>");
+	    assertCleanedJDom("<p><b^b></b=></p>", "<p><bb /></p>");
+	}
+	    
 	/**
 	 * attribute names for HTML and XML - see bug #175
 	 */
@@ -122,10 +240,13 @@ public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
 	    cleaner.getProperties().setNamespacesAware(true);
 	    
 	    // Try to quietly fix bad names with no prefixes
-	    assertCleanedHtml("<p ba;nana=\"yy\"></p>", "<p banana=\"yy\"></p>");
-	    
+	    assertCleanedDom("<p ba;nana=\"yy\"></p>", "<p banana=\"yy\"></p>");
+	    assertCleanedJDom("<p ba;nana=\"yy\"></p>", "<p banana=\"yy\" />");
+
 	    // OK - characters
 	    assertCleanedHtml("<p xx=\"yy\"></p>", "<p xx=\"yy\"></p>");
+	    assertCleanedHtml("<p value=yes></p>", "<p value=\"yes\"></p>");
+
 	    assertCleaned("<p xx=\"yy\"></p>", "<p xx=\"yy\"></p>");
 	    assertCleanedDom("<p xx=\"yy\"></p>", "<p xx=\"yy\"></p>");
 	    assertCleanedJDom("<p xx=\"yy\"></p>", "<p xx=\"yy\" />");
@@ -134,14 +255,14 @@ public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
 	    
 	    // First, lets clean them with a prefix.
 	    cleaner.getProperties().setInvalidXmlAttributeNamePrefix("hc-generated-");
-	    assertCleanedHtml("<p 1=\"yy\"></p>", "<p hc-generated-1=\"yy\"></p>");
-	    assertCleaned("<p 1=\"yy\"></p>", "<p hc-generated-1=\"yy\"></p>");
 	    assertCleanedDom("<p 1=\"yy\"></p>", "<p hc-generated-1=\"yy\"></p>");
 	    assertCleanedJDom("<p 1=\"yy\"></p>", "<p hc-generated-1=\"yy\" />");
+	    assertCleanedHtml("<p 1=\"yy\"></p>", "<p 1=\"yy\"></p>");
+	    assertCleaned("<p 1=\"yy\"></p>", "<p hc-generated-1=\"yy\"></p>");
 	    
 	    // Now, without a prefix - they have to be removed
 	    cleaner.getProperties().setInvalidXmlAttributeNamePrefix("");
-	    assertCleanedHtml("<p 1=\"yy\"></p>", "<p></p>");
+	    assertCleanedHtml("<p 1=\"yy\"></p>", "<p 1=\"yy\"></p>");
 	    assertCleaned("<p 1=\"yy\"></p>", "<p></p>");
 	    assertCleanedDom("<p 1=\"yy\"></p>", "<p></p>");
 	    assertCleanedJDom("<p 1=\"yy\"></p>", "<p />");
@@ -160,15 +281,15 @@ public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
 	    
 	    // Semicolons - OK in HTML, invalid in XML
 	    cleaner.getProperties().setInvalidXmlAttributeNamePrefix("hc-generated-");
-	    assertCleanedHtml("<p a;=\"yy\"></p>", "<p hc-generated-a=\"yy\"></p>");
-	    assertCleaned("<p a;=\"yy\"></p>", "<p hc-generated-a=\"yy\"></p>"); 
-	    assertCleanedDom("<p a;=\"yy\"></p>", "<p hc-generated-a=\"yy\"></p>"); 
-	    assertCleanedJDom("<p a;=\"yy\"></p>", "<p hc-generated-a=\"yy\" />"); 
+	    assertCleanedHtml("<p a;=\"yy\"></p>", "<p a;=\"yy\"></p>");
+	    assertCleaned("<p a;=\"yy\"></p>", "<p a=\"yy\"></p>"); 
+	    assertCleanedDom("<p a;=\"yy\"></p>", "<p a=\"yy\"></p>"); 
+	    assertCleanedJDom("<p a;=\"yy\"></p>", "<p a=\"yy\" />"); 
 
-	    assertCleanedHtml("<p ba;nana=\"yy\"></p>", "<p hc-generated-banana=\"yy\"></p>"); 
-	    assertCleaned("<p ba;nana=\"yy\"></p>", "<p hc-generated-banana=\"yy\"></p>"); 
-	    assertCleanedDom("<p ba;nana=\"yy\"></p>", "<p hc-generated-banana=\"yy\"></p>"); 
-	    assertCleanedJDom("<p ba;nana=\"yy\"></p>", "<p hc-generated-banana=\"yy\" />"); 
+	    assertCleanedHtml("<p ba;nana=\"yy\"></p>", "<p ba;nana=\"yy\"></p>"); 
+	    assertCleaned("<p ba;nana=\"yy\"></p>", "<p banana=\"yy\"></p>"); 
+	    assertCleanedDom("<p ba;nana=\"yy\"></p>", "<p banana=\"yy\"></p>"); 
+	    assertCleanedJDom("<p ba;nana=\"yy\"></p>", "<p banana=\"yy\" />"); 
 
 	    cleaner.getProperties().setAllowInvalidAttributeNames(true);
 	    assertCleanedHtml("<p a;=\"yy\"></p>", "<p a;=\"yy\"></p>"); 
@@ -178,16 +299,16 @@ public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
 	    
 	    cleaner.getProperties().setAllowInvalidAttributeNames(false);
 	    cleaner.getProperties().setInvalidXmlAttributeNamePrefix("");
-	    assertCleanedHtml("<p ba;nana=\"yy\"></p>", "<p banana=\"yy\"></p>"); 
+	    assertCleanedHtml("<p ba;nana=\"yy\"></p>", "<p ba;nana=\"yy\"></p>"); 
 	    assertCleaned("<p ba;nana=\"yy\"></p>", "<p banana=\"yy\"></p>"); 
 	    assertCleanedDom("<p ba;nana=\"yy\"></p>", "<p banana=\"yy\"></p>"); 
 	    assertCleanedJDom("<p ba;nana=\"yy\"></p>", "<p banana=\"yy\" />"); 
 	    
 	    // SOLIDUS - invalid in both
-	    assertCleanedHtml("<p 1/=\"yy\"></p>", "<p yy=\"yy\"></p>");
+	    assertCleanedHtml("<p 1/=\"yy\"></p>", "<p 1=\"1\" yy=\"yy\"></p>"); //
 	    cleaner.getProperties().setAllowInvalidAttributeNames(false);
 	    cleaner.getProperties().setInvalidXmlAttributeNamePrefix("hc-generated-");
-	    assertCleanedHtml("<p 1/=\"yy\"></p>", "<p hc-generated-1=\"1\" yy=\"yy\"></p>");
+	    assertCleanedHtml("<p 1/=\"yy\"></p>", "<p 1=\"1\" yy=\"yy\"></p>");
 	    assertCleaned("<p 1/=\"yy\"></p>", "<p hc-generated-1=\"1\" yy=\"yy\"></p>");
 	    assertCleanedDom("<p 1/=\"yy\"></p>", "<p hc-generated-1=\"1\" yy=\"yy\"></p>");
 	    assertCleanedJDom("<p 1/=\"yy\"></p>", "<p hc-generated-1=\"1\" yy=\"yy\" />");
@@ -221,6 +342,13 @@ public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
 	    assertCleaned("<p \u0000=\"yy\"></p>", "<p yy=\"yy\"></p>");	
 	    assertCleanedDom("<p \u0000=\"yy\"></p>", "<p yy=\"yy\"></p>");
 	    assertCleanedJDom("<p \u0000=\"yy\"></p>", "<p yy=\"yy\" />");
+	    
+	    // No namespaces and using colon - if the localname is null we should 
+	    // remove it
+	    cleaner.getProperties().setNamespacesAware(false);
+	    assertCleanedHtml("<p x:\u0000=\"yy\"></p>", "<p yy=\"yy\"></p>");
+	    assertCleanedHtml("<p disabled></p>", "<p disabled=\"disabled\"></p>");	
+
 	}
 	
 	@Test
@@ -668,12 +796,10 @@ public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
 	    serializer.write(node, writer, "UTF-8");
 	    assertEquals(expectedHtml, writer.toString());
 
-	    //
-	    // TODO this should also work for XML in some cases - I've commented this out for now but will return to it later.
-	    //
-	    //writer = new StringWriter();
-	    //serializer = new SimpleXmlSerializer(cleaner.getProperties());
-	    //assertEquals(expectedXml, writer.toString());
+	    writer = new StringWriter();
+	    serializer = new SimpleXmlSerializer(cleaner.getProperties());
+	    serializer.write(node, writer, "UTF-8");
+	    assertEquals(expectedXml, writer.toString());
 	}
 
 	/**
